
mtdt = LibStub("AceAddon-3.0"):NewAddon("MeantToDoThat", "AceConsole-3.0", "AceEvent-3.0")

local defaults = {
	global = {
		perChar = false,
		reciever = nil,
		output = 'smart',
		defList = {
			meant = true,
		},
		custList = {},
		msgList = {},
	},
}

local options = {
	name = "Meant To Do That",
	handler = mtdt,
	type = 'group',
	get = 'Get',
	set = 'Set',
	args = {
		opt = {
			name = 'Options',
			desc = 'Opens options dialog',
			type = 'execute',
			func = function () end,
			guiHidden = true,
			order = 0,
		},
		perChar = {
			type = 'toggle',
			name = '<NYI> Settings for character',
			desc = 'Save setting for character, not account',
			order = 10,
			cmdHidden = true,
		},
		msg = {
			type = 'input',
			name = 'Add death message',
			desc = 'Message to be sent upon death',
			width = "double",
			order = 31,
			get = nil,
			set = 'AddMessage',
			cmdHidden = true,
		},
		custList = {
			type = 'multiselect',
			name = 'Custom message list',
			desc = 'List of custom death messages.',
			width = 'double',
			get = 'GetMulti',
			set = 'SetMulti',
			values = {},
			order = 32,
			cmdHidden = true,
		},
		defList = {
			type = 'multiselect',
			name = 'Default message list',
			desc = 'List of preset death messages.',
			get = 'GetMulti',
			set = 'SetMulti',
			width = 'double',
			values = {
				meant = 'I meant to do that!',
				healer = '<NYI> %Healer% didn\'t heal me enough!',
				tank = '<NYI> %Tank% didn\'t hold agro!',
			},
			order = 33,
			cmdHidden = true,
		},
		output = {
			type = 'select',
			name = 'Output channel',
			desc = 'Channel to post message.\nSmart group will choose Instance, Raid, or Party depending on what type of group you are in.',
			values = {
				smart = 'Smart group',
				whisper = 'Whisper',
				instance = 'Instance',
				party = 'Party',
				raid = 'Raid',
				general = 'General',
			},
			order = 11,
		},
		reciever = {
			type = 'input',
			name = 'Whisper target',
			desc = 'Player to recieve your message',
			order  = 21,
			width = "double",
			-- cmdHidden = true,
		},
	},
}

function mtdt:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("MeantToDoThatDB", defaults)
	-- options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db, true)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Meant To Do That", options)
	self.profilesFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Meant To Do That")
	mtdt:RegisterChatCommand("meanttodothat", "SlashProcessorFunc")
	mtdt:RegisterChatCommand("mtdt", "SlashProcessorFunc")
	mtdt:RegisterEvent("PLAYER_DEAD")
end

function mtdt:OnEnable()
	mtdt:FetchPresence()
	options.args.custList.values = self.db.global.msgList
end

function mtdt:OnDisable()
	--
end

function mtdt:SlashProcessorFunc( input )
	if input:trim() == "opt" then
		LibStub("AceConfigDialog-3.0"):Open("Meant To Do That")
	elseif input:trim() == "test" then
		mtdt:PLAYER_DEAD()
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(mtdt, "mtdt", "Meant To Do That", input)
	end
end

function mtdt:Get( info )
	return self.db.global[info[#info]]
end

function mtdt:Set( info, value )
	self.db.global[info[#info]] = value
	if info[#info] == "reciever" then
		mtdt:FetchPresence()
	end
end

function mtdt:GetMulti( info, key )
	return self.db.global[info[#info]][key]
end

function mtdt:SetMulti( info, key, value )
	local temp = self.db.global[info[#info]]
	temp[key] = value
	self.db.global[info[#info]] = temp
end

function mtdt:AddMessage( info, value )
	self.db.global.msgList[value] = value
	self.db.global.custList[value] = true
end

function mtdt:FetchPresence()
	if (self.db.global.reciever) then
		self.reciever = GetAutoCompletePresenceID(self.db.global.reciever)
	end
end

function mtdt:PLAYER_DEAD()
	mtdt:ChooseMessage()
	mtdt:SendMessage()
end

function mtdt:ChooseMessage()
	local messages = {}
	local messageCount = 0
	for key,value in pairs(self.db.global.msgList) do
	    if self.db.global.custList[key] then
	    	messages[key] = key
	    	messageCount = messageCount + 1
	    end
	end

	for key,value in pairs(self.db.global.defList) do
	    if value then
	    	messages[key] = key
	    	messageCount = messageCount + 1
	    end
	end

	messageCount = math.random(messageCount)

	for key,value in pairs(messages) do
	    messageCount = messageCount - 1
	    if messageCount == 0 then
	    	self.msg = value
	    end
	end

	if self.msg == "meant" then
		self.msg = "I meant to do that!"
	elseif self.msg == "healer" then
		mtdt:Print("<NYI>")
	elseif self.msg == "tank" then
		mtdt:Print("<NYI>")
	end
end

function mtdt:SendMessage()
	if self.db.global.output == "whisper" then
		if self.db.global.reciever then
			if (self.reciever) then
				BNSendWhisper(self.reciever, "I meant to do that!")
			else
				SendChatMessage("I meant to do that!", WHISPER, nil, self.db.global.reciever)
			end
		else
			mtdt:Print("Please set a whisper target!")
		end
	elseif IsInGroup(LE_PARTY_CATEGORY_INSTANCE) and  (self.db.global.output == "instance" or self.db.global.output == "smart") then
		SendChatMessage("I meant to do that!", INSTANCE_CHAT)
	elseif IsInRaid() and  (self.db.global.output == "raid" or self.db.global.output == "smart") then
		SendChatMessage("I meant to do that!", RAID)
	elseif IsInGroup(LE_PARTY_CATEGORY_HOME) and  (self.db.global.output == "party" or self.db.global.output == "smart") then
		SendChatMessage("I meant to do that!", PARTY)
	elseif self.db.global.output == "general" then
		SendChatMessage("I meant to do that!", CHANNEL, nil, 1)
	else
		mtdt:Print("No suitable channel found!")
	end
end